/*
 * Copyright (c) 2005, 2006 Sharp Corporation
 */

#include "shdfb_gfx.h"

#include <core/system.h>

extern DisplayLayerFuncs  tShdfbOldPrimaryFuncs;
extern void *pShdfbOldPrimaryDriverData;

static DFBResult
shdfbARGBPortraitInitLayer (CoreLayer *ptLayer,
                    void *pDriver_data,
                    void *pLayer_data,
                    DFBDisplayLayerDescription *ptDescription,
                    DFBDisplayLayerConfig *ptConfig,
                    DFBColorAdjustment *ptAdjustment)
{
    DFBResult tRet;
    dShdfbDriverData_t *ptDriver = (dShdfbDriverData_t *) pDriver_data;
    int32 iRet = 0, iWidth;
	
    tRet = tShdfbOldPrimaryFuncs.InitLayer (ptLayer, pShdfbOldPrimaryDriverData, pLayer_data, ptDescription, ptConfig, ptAdjustment);
	if (tRet) {
		return tRet;
	}

	ptDescription->caps = DLCAPS_SURFACE;
    ptDescription->type = DLTF_GRAPHICS | DLTF_STILL_PICTURE;
    snprintf(ptDescription->name, DFB_DISPLAY_LAYER_DESC_NAME_LENGTH, "SHDFB ARGB Portrait plane");

    ptDescription->caps |= DLCAPS_ALPHACHANNEL;
    ptConfig->pixelformat = ARGB_PLANE_FORMAT;
	
#ifndef EMULATION
	if (ptDriver->tARGB_PAddr.pdvAddr == NULL) {
	    iRet = iLnxDFBInitSubPortrait (&ptDriver->tARGB_PAddr);
	}
#else
	iRet = 0;
#endif    

   	ptConfig->width = ptDriver->tARGB_PAddr.uiWidth;
   	ptConfig->height = ptDriver->tARGB_PAddr.uiHeight;

    return iRet == 0 ? DFB_OK : DFB_FAILURE;
}


DFBResult
shdfbARGBPortraitAllocateSurface (CoreLayer *ptLayer,
								  void *pvDriverData,
								  void *pvLayerData,
								  void *pvRegionData,
								  CoreLayerRegionConfig *ptConfig,
								  CoreSurface **ptRetSurface)
{
    DFBResult tRet;
    dShdfbDriverData_t *ptDriver = (dShdfbDriverData_t *) pvDriverData;
    CoreSurface *ptSurface;
    
    tRet =  tShdfbOldPrimaryFuncs.AllocateSurface (ptLayer, pShdfbOldPrimaryDriverData, pvLayerData, pvRegionData, ptConfig, ptRetSurface);
    if (tRet) {
    	return tRet;
    }
    
    ptSurface = *ptRetSurface;

#ifndef EMULATION
    ptSurface->front_buffer->system.addr = (void *) &ptDriver->tARGB_PAddr;
    ptSurface->front_buffer->video.offset = 4;
#else    
    ptSurface->front_buffer->system.addr = (void *) ptDriver->tARGB_PAddr.pdvAddr;
    fusion_reactor_attach (ptSurface->object.reactor, shdfbStopSurface, ptSurface, &ptDriver->reaction[ptDriver->reaction_num++]);
#endif	

    return DFB_OK;
}
	
DisplayLayerFuncs tShdfbARGBPortraitFuncs = {
    LayerDataSize:      shdfbARGBdatasize,
    InitLayer:          shdfbARGBPortraitInitLayer,
    SetRegion:          shdfbSetRegion,
    RemoveRegion:        shdfbRemoveRegion,
    TestRegion:         shdfbARGBTestRegion,
    AllocateSurface: shdfbARGBPortraitAllocateSurface,
    ReallocateSurface: shdfbReallocateSurface,
};
